@@ -74,6 +74,8 @@ sub _compile_enclosed {
     \@enclosed;
 }
 
+sub display_name { shift->name }
+
 sub in {
     my ($self, $value) = @_;
 
@@ -164,6 +166,10 @@ Returns parameter description.
 
 Returns parameter name.
 
+=head3 display_name
+
+An alias to L<Raisin::Param/name>.
+
 =head3 named
 
 Returns C<true> if it's path parameter.
