@@ -5,31 +5,72 @@ use warnings;
 
 use parent 'Raisin::Plugin';
 
+use Carp 'croak';
 use Data::Dumper;
 use Digest::MD5 qw/md5_hex/;
 use JSON qw/encode_json/;
+use List::Util qw/pairmap/;
 
-my %SETTINGS;
 my %DEFAULTS;
+my %SETTINGS;
+
+my $HTTP_OK = 200;
 
 sub build {
-    my ($self, %args) = @_;
-
-    # Enable CORS
-    if ($args{enable} && lc($args{enable}) eq 'cors') {
-        $self->app->add_middleware(
-            'CrossOrigin',
-            origins => '*',
-            methods => [qw(GET HEAD POST DELETE PUT PATCH OPTIONS)],
-            headers => [qw(api_key authorization content-type accept)]
-        );
-    }
+    my $self = shift;
 
     $self->register(
         swagger_build_spec => sub { $self->_spec_20 },
         swagger_setup => sub { %SETTINGS = @_ },
+        swagger_security => \&swagger_security,
     );
-    $self->{swagger_version} = '2.0';
+
+    1;
+}
+
+sub swagger_security {
+    my %p = @_;
+
+    croak 'Invalid `type`' unless grep { $p{type} eq $_ } qw/basic api_key oauth2/;
+
+    my %security;
+
+    if ($p{type} eq 'basic') {
+        $security{ $p{name} } = {
+            type => 'basic',
+        };
+    }
+    elsif ($p{type} eq 'api_key') {
+        croak 'Invalid `in`' unless grep { $p{in} eq $_ } qw/query header/;
+
+        $security{ $p{name} } = {
+            type => 'apiKey',
+            name => $p{name},
+            in => $p{in},
+        };
+    }
+    elsif ($p{type} eq 'oauth2') {
+        croak 'Invalid `flow`' unless grep { $p{flow} eq $_ } qw/implicit password application accessCode/;
+
+        $security{ $p{name} } = {
+            type => 'oauth2',
+            flow => $p{flow},
+            scopes => $p{scopes},
+        };
+
+        if (grep { $p{flow} eq $_ } qw/implicit accessCode/) {
+            $security{ $p{name} }{authorizationUrl} = $p{authorization_url};
+        }
+
+        if (grep { $p{flow} eq $_ } qw/password application accessCode/) {
+            $security{ $p{name} }{tokenUrl} = $p{token_url};
+        }
+    }
+
+    $SETTINGS{security} = {
+        %{ $SETTINGS{security} || {} },
+        %security,
+    };
 }
 
 sub _spec_20 {
@@ -51,21 +92,21 @@ sub _spec_20 {
     $DEFAULTS{produces} = \@content_types;
 
     my %spec = (
-        swagger  => '2.0',
-        info     => _info_object($app),
-        host     => $req->env->{HTTP_HOST},
+        swagger => '2.0',
+        info => _info_object($app),
+        host => $req->env->{SERVER_NAME} || $req->env->{HTTP_HOST},
         basePath => $base_path,
-        schemes  => [$req->scheme],
+        schemes => [$req->scheme],
         consumes => \@content_types,
         produces => \@content_types,
-        paths    => _paths_object($routes), #R
+        paths => _paths_object($routes),
         definitions => _definitions_object($routes),
         #parameters => undef,
         #responses => undef,
-        #securityDefinitions => undef,
-        #security => undef,
+        securityDefinitions => _security_definitions_object(),
+        security => _security_object(),
         tags => _tags_object($self->app),
-        #externalDocs => '', # TODO
+        #externalDocs => undef,
     );
 
     # routes
@@ -93,7 +134,7 @@ sub _contact_object {
 sub _license_object {
     my $license = shift;
     my %obj = (
-        name => $license->{name}, #R
+        name => $license->{name},
     );
     $obj{url} = $license->{url} if $license->{url};
     \%obj;
@@ -103,8 +144,8 @@ sub _info_object {
     my $app = shift;
 
     my %obj = (
-        title => $SETTINGS{title} || 'API', #R
-        version => $app->api_version || '0.0.1', #R
+        title => $SETTINGS{title} || 'API',
+        version => $app->api_version || '0.0.1',
     );
 
     $obj{description} = $SETTINGS{description} if $SETTINGS{description};
@@ -116,46 +157,113 @@ sub _info_object {
     \%obj;
 }
 
+sub _security_object {
+    my @obj = map { { $_->{name} => $_->{scopes} || [] } } values %{ $SETTINGS{security} };
+    \@obj;
+}
+
+sub _security_definitions_object { $SETTINGS{security} || {} }
+
+sub _paths_object {
+    my $routes = shift;
+
+    my %obj;
+    for my $r (sort { $a->path cmp $b->path } @$routes) {
+        next if lc($r->method) eq 'options';
+
+        my $path = $r->path;
+        $path =~ s#:([^/]+)#{$1}#msix;
+
+        $obj{ $path }{ lc($r->method) } = _operation_object($r);
+    }
+
+    \%obj;
+}
+
+sub _operation_object {
+    my $r = shift;
+
+    my $path = $r->path;
+    $path =~ tr#/:#_#;
+    my $operation_id = lc($r->method) . $path;
+
+    my %obj = (
+        consumes => $DEFAULTS{consumes},
+        #deprecated => 'false',
+        description => $r->desc || '',
+        #externalDocs => '',
+        operationId => $operation_id,
+        produces => $DEFAULTS{produces},
+        responses => {
+            default => {
+                description => 'Unexpected error',
+                #examples => '',
+                #headers => '',
+                #schema => '',
+            },
+            # Adds a response object from route's entity if it exists
+            %{ _response_object($r) },
+        },
+        #schemes => [],
+        #security => {}, # TODO per operation permissions
+        summary => $r->summary || '',
+        tags => $r->tags,
+    );
+
+    my $params = _parameters_object($r->method, $r->params);
+    $obj{parameters} = $params if scalar @$params;
+
+    \%obj;
+}
+
+sub _response_object {
+    my $r = shift;
+    return {} unless $r->entity;
+
+    my $name = $r->entity;
+
+    my %obj = (
+        $HTTP_OK => {
+            description => $r->desc || $r->summary || '',
+            schema => {
+                '$ref' => sprintf('#/definitions/%s', _name_for_object($name)),
+            }
+        },
+    );
+
+    \%obj;
+}
+
 sub _parameters_object {
     my ($method, $pp) = @_;
 
     my @obj;
     for my $p (@$pp) {
-        my ($type, $format) = _param_type($p->type);
+        my ($type) = _param_type($p->type);
 
         # Available: query, header, path, formData or body
         my $location = do {
             if    ($p->in)                       { $p->in }
-            elsif ($p->named)                    {'path'}
-            elsif ($type eq 'object')            {'body'}
-            elsif ($method =~ /patch|post|put/i) {'formData'}
-            else                                 {'query'}
+            elsif ($p->named)                    { 'path' }
+            elsif ($type eq 'object')            { 'body' }
+            elsif ($method =~ /patch|post|put/i) { 'formData' }
+            else                                 { 'query' }
         };
 
+        my $ptype = _param_type_object($p);
+        if ($p->type->name eq 'HashRef') {
+            $ptype->{schema}{'$ref'} = delete $ptype->{'$ref'};
+        }
+
         my %param = (
-            description => $p->desc || "",
-            in          => $location, #R
-            name        => $p->name, #R
+            description => $p->desc || '',
+            in          => $location,
+            name        => $p->name,
             required    => $p->required ? JSON::true : JSON::false,
+            %$ptype,
         );
         $param{default} = $p->default if defined $p->default;
 
-        if ($type eq 'object') {
-            $param{schema} = {
-                '$ref' => sprintf('#/definitions/%s', _name_for_object($p)),
-            };
-        }
-        elsif ($type eq 'array') {
-            $param{type} = $type;
-
-            my ($items_type, $items_format) = _param_type($p->type->type_parameter);
-            $param{items}{type} = $items_type;
-            $param{items}{format} = $items_format if $items_format;
-        }
-        else {
-            $param{type} = $type; #R
-            $param{format} = $format if $format;
-        }
 
         push @obj, \%param;
     }
@@ -168,8 +276,11 @@ sub _definitions_object {
 
     my @objects;
     for my $r (@$routes) {
-        my @pp = @{ $r->params };
+        if ($r->entity) {
+            push @objects, $r->entity;
+        }
 
+        my @pp = @{ $r->params };
         while (my $p = pop @pp) {
             next if $p->type->name ne 'HashRef';
 
@@ -189,25 +300,9 @@ sub _schema_object {
     my (@required, %properties);
 
     for my $pp (@{ $p->enclosed }) {
+        $properties{ $pp->display_name } = _param_type_object($pp);
 
-        if ($pp->type->name eq 'HashRef') {
-            $properties{ $pp->name }{'$ref'} =
-                sprintf('#/definitions/%s', _name_for_object($pp));
-        }
-        elsif ($pp->type->display_name =~ /^ArrayRef/) {
-            $properties{ $pp->name }{type} = 'array';
-
-            my ($items_type, $items_format) = _param_type($pp->type->type_parameter);
-            $properties{ $pp->name }{items}{type} = $items_type;
-            $properties{ $pp->name }{items}{format} = $items_format if $items_format;
-        }
-        else {
-            my ($type, $format) = _param_type($pp->type);
-            $properties{ $pp->name }{type} = $type;
-            $properties{ $pp->name }{format} = $format if $format;
-        }
-
-        push @required, $pp->name if $pp->required;
+        push @required, $pp->display_name if $pp->required;
     }
 
     my %object = (
@@ -221,59 +316,6 @@ sub _schema_object {
     \%object;
 }
 
-sub _operation_object {
-    my $r = shift;
-
-    my $path = $r->path;
-    $path =~ tr#/:#_#;
-    my $operation_id = lc($r->method) . $path;
-
-    my %obj = (
-        tags => $r->tags,
-        summary => $r->summary || "",
-        description => $r->desc || "",
-        #externalDocs => '',
-        operationId => $operation_id,
-        consumes => $DEFAULTS{consumes},
-        produces => $DEFAULTS{produces},
-        # TODO:
-        responses => {
-            500 => {
-                description => 'Server exception',
-                #schema => '',
-                #headers => '',
-                #examples => '',
-            },
-            400 => {
-                description => 'Invalid parameters',
-            }
-        },
-        #schemes => [],
-        #deprecated => 'false', # TODO
-        #security => '',
-    );
-
-    my $params = _parameters_object($r->method, $r->params);
-
-    $obj{parameters} = $params if scalar @$params;
-
-    \%obj;
-}
-
-sub _paths_object {
-    my $routes = shift;
-
-    my %obj;
-    for my $r (sort { $a->path cmp $b->path } @$routes) {
-        my $path = $r->path;
-        $path =~ s#:([^/]+)#{$1}#msix;
-
-        $obj{ $path }{ lc($r->method) } = _operation_object($r);
-    }
-
-    \%obj;
-}
-
 sub _tags_object  {
     my $app = shift;
 
@@ -285,7 +327,7 @@ sub _tags_object  {
     my @tags;
     for my $t (keys %tags) {
         my $tag = {
-            name => $t, #R
+            name => $t,
             description => $app->resource_desc($t),
             #externalDocs => {
             #    description => '',
@@ -298,6 +340,42 @@ sub _tags_object  {
     \@tags;
 }
 
+sub _param_type_object {
+    my $p = shift;
+
+    my %item;
+
+    if ($p->type->name eq 'HashRef') {
+        $item{'$ref'} = sprintf('#/definitions/%s', _name_for_object($p));
+    }
+    elsif ($p->type->display_name =~ /^ArrayRef/) {
+        $item{type} = 'array';
+
+        my ($type, $format) = _param_type($p->type->type_parameter);
+        if ($type eq 'object') {
+            my $ref = do {
+                if   ($p->can('using') && $p->using) { $p->using }
+                else { $p }
+            };
+
+            $item{items}{'$ref'} = sprintf('#/definitions/%s', _name_for_object($ref));
+        }
+        else {
+            $item{type} = $type;
+            $item{format} = $format if $format;
+            $item{description} = $p->desc if $p->desc;
+        }
+    }
+    else {
+        my ($type, $format) = _param_type($p->type);
+        $item{type} = $type;
+        $item{format} = $format if $format;
+        $item{description} = $p->desc if $p->desc;
+    }
+
+    \%item;
+}
+
 sub _param_type {
     my $t = shift;
 
@@ -320,11 +398,6 @@ sub _param_type {
 
 sub _name_for_object {
     my $obj = shift;
-    sprintf '%s-%s', ucfirst($obj->name), _fingerprint($obj);
-}
-
-sub _fingerprint {
-    my $obj = shift;
 
     local $Data::Dumper::Deparse = 1;
     local $Data::Dumper::Indent = 0;
@@ -333,9 +406,10 @@ sub _fingerprint {
     local $Data::Dumper::Sortkeys = 1;
     local $Data::Dumper::Terse = 1;
 
-    uc(substr(md5_hex(Data::Dumper->Dump([$obj], [qw/obj/])), 0, 10));
-}
+    my $fingerprint = md5_hex(Data::Dumper->Dump([$obj], [qw/obj/]));
 
+    sprintf '%s-%s', ucfirst($obj->name), uc(substr($fingerprint, 0, 10));
+}
 
 1;
 
@@ -351,29 +425,104 @@ Raisin::Plugin::Swagger - Generate API documentation.
 
 =head1 DESCRIPTION
 
-Generates a L<Swagger|https://github.com/wordnik/swagger-core>
-compatible API documentaion.
+Generates an API description of application.
 
-Provides a documentation by C</swagger.json> URL.
-You can use this url in L<Swagger UI|http://petstore.swagger.io/>.
+=head1 SPECIFICATION
+
+Compatible with
+L<Swagger|http://swagger.io/>/L<OpenAPI|https://www.openapis.org/> Spec 2.0.
 
 =head1 CORS
 
-Enables a cross-origin resource sharing.
+To enable L<Cross-Origin HTTP Requests|https://developer.mozilla.org/en/docs/Web/HTTP/Access_control_CORS>
+you should enable a L<Plack::Middleware::CrossOrigin> middleware with all the
+parameters you need (like origins, methods, headers, etc.).
 
-    plugin 'Swagger', enable => 'CORS';
+    middleware 'CrossOrigin',
+        origins => '*',
+        methods => [qw/DELETE GET HEAD OPTIONS PATCH POST PUT/],
+        headers => [qw/accept authorization content-type api_key_token/];
 
-=head1 VERSION
+Alternatively you can set CORS headers in a L<before|Raisin/HOOKS> hook.
 
-Supports only version 2.0 of Swagger.
+=head1 SECURITY
 
+L<Raisin> has a limited support of OpenAPI security objects. To make it generate
+security objects configure it in the way it described below.
+
+Add a B<api_key> security via B<stoken> header.
+
+    swagger_security(name => 'stoken', in => 'header', type => 'api_key');
+
+Add the header name to L<Raisin::Plugin::Swagger/CORS> headers if you use B<api_key>.
+
+    middleware 'CrossOrigin',
+        origins => '*',
+        methods => [qw/DELETE GET HEAD OPTIONS PATCH POST PUT/],
+        headers => [qw/stoken accept authorization content-type/];
+
+=head3 Limitations
+
+=over
+
+=item * L<Raisin> doesn't support per operation security.
+
+=item * L<Raisin> doesn't support B<oauth2>, only B<basic> and B<api_key> are supported.
+
+=back
+
+=head3 Example Application
+
+Example of a secured application.
+
+    use strict;
+    use warnings;
+
+    middleware '+MyAuthMiddleware';
+
+    middleware 'CrossOrigin',
+        origins => '*',
+        methods => [qw/DELETE GET HEAD OPTIONS PATCH POST PUT/],
+        headers => [qw/stoken accept authorization content-type/];
+
+    plugin 'Swagger';
+    swagger_security(name => 'stoken', in => 'header', type => 'api_key');
+
+    get sub { { data => 'ok' } };
+
+    run;
+
+Example of a middleware used in the application.
+
+    package Auth;
+
+    use strict;
+    use warnings;
+
+    use parent 'Plack::Middleware';
+    use Plack::Request;
+
+    sub call {
+        my ($self, $env) = @_;
+
+        my $req = Plack::Request->new($env);
+
+        if (($req->header('stoken') // '') eq 'secret' || $req->path eq '/swagger.json') {
+            $self->app->($env);
+        }
+        else {
+            [403, [], ['forbidden']];
+        }
+    }
+
+    1;
 
 =head1 FUNCTIONS
 
 =head3 swagger_setup
 
-The function configures base OpenAPI paramters, be aware it is not validating
-and will be passed as is to OpenAPI client.
+Configures base OpenAPI parameters, be aware they aren't validating
+and passing to the specification as is.
 
 Properly configured section has following attributes:
 B<title>, B<description>, B<terms_service>, B<contact> and B<license>.
@@ -382,8 +531,6 @@ B<Contact> has B<name>, B<url>, B<email>.
 
 B<License> has B<name> and B<url>.
 
-See an example below.
-
     swagger_setup(
         title => 'BatAPI',
         description => 'Simple BatAPI.',
@@ -400,6 +547,14 @@ See an example below.
         },
     );
 
+=head3 swagger_security
+
+Configures OpenAPI security options.
+
+Allowed types are B<basic>, B<api_key> and B<oauth2>.
+
+For more information please check OpenAPI specification and L<Raisin::Plugin::Swagger/SECURITY>.
+
 =head1 AUTHOR
 
 Artur Khabibullin - rtkh E<lt>atE<gt> cpan.org
